{{
*************************************
* File jvmStack: handling the stack *
*************************************
}}

{{
   Values are stored onto the stack using big endian.
   The jvmData methods writeJavaIntBE and readJavaIntBE are used to read/write stack items.
   The stack grows upwards.
   Stackpointer points to first free stack byte.
}}
{{
  stackframe just before invoke       stackframe just after invoke       stackframe just after return
                                      (only if method not native)

       |             |                     |             |                    |             |
       +-------------+                     +-------------+ <- fp              +-------------+
       | aref / arg0 |                     | aref / arg0 |                    | return val  |
       +-------------+                     +-------------+                    +-------------+ <- sp
       | arg0 / arg1 |                     | arg0 / arg1 |
       |    .....    |                     |    .....    |
       +-------------+ <- sp               +-------------+
                                           |   locals    |
                                           |    .....    |
                                           +-------------+
                                           | old method  |
                                           +-------------+
                                           |   old fp    |
                                           +-------------+
                                           | return addr |
                                           +-------------+ <- sp
                                           
}}



OBJ
  jd: "jvmData"

  
PUB push(value) | sp
{{ Auxiliary function (not bytecode): Meaning: Push word onto the stack. }}
  sp := jd.getJvmSP
  jd.writeJavaIntBE(sp,value)
  jd.setJvmSP(sp+2)

  
PUB pop: value | sp
{{ Auxiliary function (not bytecode): Meaning: Pop word from the stack. }}
  sp := jd.getJvmSP - 2
  jd.setJvmSP(sp)
  return jd.readJavaIntBE(sp)


PUB jb_nop
{{ Meaning: Do nothing. }}
{{
  JEM format: nop
}}
  jd.incJvmPC(1)
  

PUB jb_pop: value | val
{{ Meaning: Pop the top word from the stack. }}
{{
  JEM format: pop
}}
  val := pop
  jd.incJvmPC(1)
  return val
    

PUB jb_pop2: value | w1,w2
{{ Meaning: Pop the top two words from the stack. }}
{{
  JEM format: pop2
}}
  w1 := pop  'lsw
  w2 := pop  'msw
  jd.incJvmPC(1)
  return (w2 << 16) | w1  'return 32bits result

  
PUB jb_dup | w1
{{ Meaning: Duplicate the top word on the stack. }}
{{
  JEM format: dup
}}
  w1 := pop
  push(w1)
  push(w1)
  jd.incJvmPC(1)
  

PUB jb_dup2 | w1,w2
{{ Meaning: Duplicate the top two words on the stack. }}
{{
  JEM format: dup2
}}
  w1 := pop
  w2 := pop
  push(w2)
  push(w1)
  push(w2)
  push(w1)
  jd.incJvmPC(1)
  
  
PUB jb_dup_x1 | w1,w2
{{ Meaning: Duplicate the top word on the stack and insert a copy two words down in the stack. }}
{{
  JEM format: dup_x1
}}
  w1 := pop
  w2 := pop
  push(w1)
  push(w2)
  push(w1)
  jd.incJvmPC(1)
  
  
PUB jb_dup2_x1 | w1,w2,w3
{{ Meaning: Duplicate the top two words on the stack and insert the copies two words down in the stack. }}
{{
  JEM format: dup2_x1
}}
  w1 := pop
  w2 := pop
  w3 := pop
  push(w2)
  push(w1)
  push(w3)
  push(w2)
  push(w1)
  jd.incJvmPC(1)
  

PUB jb_dup_x2 | w1,w2,w3
{{ Meaning: Duplicate the top word on the stack and insert the copy three words down in the stack. }}
{{
  JEM format: dup_x2
}}
  w1 := pop
  w2 := pop
  w3 := pop
  push(w1)
  push(w3)
  push(w2)
  push(w1)
  jd.incJvmPC(1)
  

PUB jb_dup2_x2 | w1,w2,w3,w4
{{ Meaning: Duplicate the top two words on the stack and insert the copies three words down in the stack. }}
{{
  JEM format: dup2_x2
}}
  w1 := pop
  w2 := pop
  w3 := pop
  w4 := pop
  push(w2)
  push(w1)
  push(w4)
  push(w3)
  push(w2)
  push(w1)
  jd.incJvmPC(1)
  

PUB jb_swap | w1,w2
{{ Meaning: Swap the top two elements on the stack. }}
{{
  JEM format: swap
}}
  w1 := pop
  w2 := pop
  push(w1)
  push(w2)
  jd.incJvmPC(1)
  
       